/*
*  C Implementation: dateclient
*
* Description: connects to a server, reads till end of stream, exits
*
*
* Author: Konrad Rosenbaum <konrad@silmor.de>, (C) 2006
*
* Copyright: See COPYING file that comes with this distribution
*
*/

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

int main(int argc,char**argv)
{
	int port,sock,ge;
	char*tmp;
	struct sockaddr_in6 sa;
	struct addrinfo hints;
	struct addrinfo *result;
	/*check arguments*/
	if(argc!=3){
		fprintf(stderr,"Usage: %s <host> <port>\n",*argv);
		return 1;
	}
	port=strtol(argv[2],&tmp,0);
	if(*tmp!=0 || port<1 || port>65535){
		fprintf(stderr,"Port must be a number in the range 1<=x<=65535!\n");
		return 1;
	}
	/*retrieve IP from host name*/
	memset(&hints,0,sizeof(hints));
	hints.ai_family=PF_INET6;
	ge=getaddrinfo(argv[1],0,&hints,&result);
	if(ge!=0){
		fprintf(stderr,"Unable to resolve address: %s.\n",gai_strerror(ge));
		return 1;
	}
	if(result==0){
		fprintf(stderr,"Name resolution returned NULL result.\n");
		return 1;
	}
	if(result->ai_addr==0){
		fprintf(stderr,"Name resolution did not return an address.\n");
		return 1;
	}
	if(result->ai_family!=PF_INET6){
		fprintf(stderr,"Name resulution returned address info of non-inet6 type %i.\n",result->ai_family);
		return 1;
	}
	/*create socket*/
	sock=socket(PF_INET6,SOCK_STREAM,0);
	if(sock<0){
		fprintf(stderr,"Failed to retrieve socket: %s\n",strerror(errno));
		return 1;
	}
	/*connect it to host/port*/
	memset(&sa,0,sizeof(sa));
	sa.sin6_family=AF_INET6;
	sa.sin6_addr=((struct sockaddr_in6*)result->ai_addr)->sin6_addr;
	sa.sin6_port=htons(port);
	freeaddrinfo(result);
	if(connect(sock,&sa,sizeof(sa))!=0){
		fprintf(stderr,"Unable to connect: %s.\n",strerror(errno));
		return 1;
	}
	/*loop: read data, dump it on stdout*/
	for(;;){
		int r;
		char buf[1024];
		r=read(sock,buf,sizeof(buf));
		if(r==0)break;
		if(r<0){
			fprintf(stderr,"Error while reading: %s\n",strerror(errno));
			break;
		}
		write(STDOUT_FILENO,buf,r);
	}
	/*close socket*/
	close(sock);
	/*success*/
	return 0;
}